local helpers = require("null-ls.helpers")
local methods = require("null-ls.methods")

local DIAGNOSTICS = methods.internal.DIAGNOSTICS
local FORMATTING = methods.internal.FORMATTING

local M = {}

M.quick_lint_js = helpers.make_builtin({
  name = "quick_lint_js",
  meta = {
    url = "https://quick-lint-js.com/",
    description = "quick-lint-js finds bugs in JavaScript programs.",
    notes = {
      "[160× faster than ESLint](https://quick-lint-js.com/benchmarks/), quick-lint-js gives you instant feedback as you code.",
      "Find bugs in your JavaScript before your finger leaves the keyboard.",
      "Lint any JavaScript file with no configuration.",
      "See https://quick-lint-js.com/install/ for the installation documentation.",
    },
  },
  method = DIAGNOSTICS,
  filetypes = { "javascript", "javascriptreact" },
  generator_opts = {
    command = "quick-lint-js",
    args = { "--output-format", "vim-qflist-json", "--stdin" },
    to_stdin = true,
    format = "json",
    check_exit_code = function(c)
      return c <= 1
    end,
    on_output = function(params)
      local diags = {}
      for _, d in ipairs(params.output.qflist) do
        table.insert(diags, {
          row = d.lnum,
          end_row = d.end_lnum,
          col = d.col,
          end_col = d.end_col + 1,
          code = d.nr,
          message = d.text,
          severity = d.type,
        })
      end
      return diags
    end,
  },
  factory = helpers.generator_factory,
})

M.lispformat = helpers.make_builtin({
  name = "lispformat",
  meta = {
    url = "https://github.com/eschulte/lisp-format",
    description = "A tool to format lisp code. Designed to mimic clang-format.",
    notes = {
      "This requires a working installation of vanilla Emacs. To install Emacs, refer to the [installation instructions](https://www.gnu.org/software/emacs/manual/html_node/efaq/Installing-Emacs.html) or use a package manager.",
    },
  },
  method = FORMATTING,
  filetypes = { "lisp" },
  generator_opts = {
    command = "lisp-format",
    args = { "$FILENAME" },
    to_stdin = true,
  },
  factory = helpers.formatter_factory,
})

return M
