local null_ls = require("null-ls")
local null_ls_extras = require("config.null-ls-extras")

-- if you want to set up formatting on save, you can use this as a callback
local augroup = vim.api.nvim_create_augroup("LspFormatting", {})

-- add to your shared on_attach callback
local on_attach = function(client, bufnr)
  if client.supports_method("textDocument/formatting") then
    vim.api.nvim_clear_autocmds({ group = augroup, buffer = bufnr })
    vim.api.nvim_create_autocmd("BufWritePre", {
      group = augroup,
      buffer = bufnr,
      callback = function()
        vim.lsp.buf.format({
          filter = function(c)
            return c.name == "null-ls"
          end,
          bufnr = bufnr,
        })
      end,
    })
  end
end

null_ls.setup({
  timeout = -1,
  on_attach = on_attach,
  sources = {
    -- null_ls_extras.quick_lint_js,
    null_ls.builtins.diagnostics.shellcheck,
    null_ls.builtins.formatting.bibclean,
    null_ls.builtins.formatting.black,
    null_ls.builtins.formatting.isort,
    -- null_ls_extras.lispformat,
    null_ls.builtins.formatting.prettier,
    null_ls.builtins.formatting.shellharden,
    null_ls.builtins.formatting.shfmt,
    null_ls.builtins.formatting.styler,
    null_ls.builtins.formatting.stylua,
  },
  debug = true,
})
