require("packer").startup(function()
  use("wbthomason/packer.nvim")

  -- core plugins
  use({ "junegunn/fzf" })
  use({ "junegunn/fzf.vim" })

  -- opt plugins
  use({ "lervag/vimtex", ft = { "tex" } })
  use({ "gpanders/nvim-parinfer", ft = { "lisp", "scheme", "racket", "clojure" } })
  use({ "norcalli/nvim-colorizer.lua", cmd = "ColorizerToggle" })
  use({ "monkoose/fzf-hoogle.vim", ft = "haskell" })
  use({
    "numToStr/Comment.nvim",
    keys = { "gcc", "gc" },
    config = function()
      require("Comment").setup()
    end,
  })
  use({
    "nvim-treesitter/nvim-treesitter",
    run = ":TSUpdate",
    event = "BufRead",
    requires = {
      "nvim-treesitter/nvim-treesitter-textobjects",
      after = "nvim-treesitter",
      config = function()
        require("config.treesitter")
      end,
    },
  })
  use({
    "neovim/nvim-lspconfig",
    ft = {
      "c",
      "clojure",
      "cpp",
      "haskell",
      "javascript",
      "julia",
      "mma",
      "purescript",
      "python",
      "racket",
      "rust",
      "typescript",
    },
    config = function()
      require("config.lsp")
      require("config.diagnostics")
    end,
  })
  use({
    "jose-elias-alvarez/null-ls.nvim",
    ft = { "python", "lisp", "lua", "css", "html", "sh", "bib", "markdown", "r" },
    requires = {
      "nvim-lua/plenary.nvim",
      after = "null-ls.nvim",
      config = function()
        require("config.null-ls")
        require("config.diagnostics")
      end,
    },
  })
  use({
    "danymat/neogen",
    config = function()
      require("neogen").setup()
    end,
    ft = { "python", "rust", "c", "cpp", "javascript", "sh", "typescript" },
    requires = "nvim-treesitter/nvim-treesitter",
    tag = "*",
  })
  use({
    "kosayoda/nvim-lightbulb",
    commit = "56b9ce3",
    ft = {
      "c",
      "clojure",
      "cpp",
      "haskell",
      "javascript",
      -- "julia",
      "purescript",
      -- "racket",
      "rust",
      "typescript",
    },
    config = function()
      require("nvim-lightbulb").setup({
        sign = { enabled = true },
        -- virtual_text = { enabled = true, text = "▮", hl_mode = "combine" },
        autocmd = { enabled = true, events = { "CursorHold", "CursorHoldI" } },
      })
    end,
  })
  use({
    "p00f/clangd_extensions.nvim",
    ft = { "c", "cpp" },
    after = "nvim-lspconfig",
    config = function()
      require("config.clangd-extensions")
    end,
  })
  use({
    "simrat39/rust-tools.nvim",
    ft = { "rust" },
    config = function()
      require("config.rust-tools")
    end,
  })
end)
