vim.g.loaded_perl_provider = 0
vim.g.loaded_ruby_provider = 0
vim.g.loaded_python_provider = 0
vim.g.netrw_banner = 0
vim.g.python3_host_prog = "/usr/bin/python3"
vim.g.python3_host_skip_check = 1
vim.g.vimtex_view_method = "zathura"

vim.bo.expandtab = true -- use spaces instead of tabs
vim.bo.formatoptions = "crqnj" -- automatic formatting options
vim.bo.shiftwidth = 2 -- size of an indent
vim.bo.smartindent = true -- insert indents automatically
vim.bo.tabstop = 2 -- number of spaces tabs count for
vim.bo.textwidth = 80 -- maximum width of text

vim.o.clipboard = "unnamedplus" -- use system copy/paste
vim.o.hidden = true -- enable modified buffers in background
vim.o.hlsearch = true -- highlight search results
vim.o.ignorecase = true -- ignore case
vim.o.inccommand = "nosplit" -- disable preview for :substitute
vim.o.incsearch = true -- show search matches as you type
vim.o.joinspaces = false -- no double spaces with join after a dot
vim.o.laststatus = 3 -- unified status line
vim.o.lazyredraw = true -- for macros and regex stuff
vim.o.listchars = "tab:▸ ,eol:¬,trail:·,extends:⣿,precedes:⣿"
vim.o.matchtime = 3 -- matching parenthesis
vim.o.mouse = "nv" -- allow mouse actions
vim.o.shiftround = true -- move in multiples of shiftwidth with < and >
vim.o.shell = "/bin/bash --rcfile /home/chl/bin/nvim-bashrc" -- fast shell
vim.o.shortmess = "I" -- no startup screen
vim.o.showmode = false -- don't show mode switch message
vim.o.smartcase = true -- ignore case if search pattern is lowercase
vim.o.splitbelow = true -- horizontal split direction
vim.o.splitright = true -- vertical split direction
vim.o.termguicolors = true -- true color support
vim.o.thesaurus = "~/.local/share/nvim/mthesaur.txt"
vim.o.undofile = true
vim.o.updatetime = 300 -- timer (swap + LSP stuff)
vim.o.wildmenu = true -- enhanced menu
vim.o.wildmode = "longest,full" -- command-line completion mode

vim.opt.dictionary:append("/usr/share/dict/words")
vim.opt.iskeyword:append({ "-" }) -- Lisp/Scheme/Bibtex (see set lisp)
vim.opt.wildignore:append({ "*swp", "*.class", "*.pyc", "*.o", "*.obj", "*.so" })
vim.opt.fillchars = {
  diff = "⣿",
  fold = "-",
  eob = " ",
  foldopen = "▾",
  foldsep = "│",
  foldclose = "▸",
}

vim.wo.cursorline = true -- highlight current line number
vim.wo.list = false -- do not show invisible characters by default
vim.wo.number = false -- show line numbers
vim.wo.relativenumber = false -- show relative line numbers
vim.wo.signcolumn = "auto:1" -- don't show sign column

vim.cmd([[set grepprg=rg\ --vimgrep\ --no-heading\ --smart-case\ --hidden\ --glob\ !.git]])
vim.cmd([[set grepformat^=%f:%l:%c:%m]])
vim.api.nvim_exec([[ let g:fzf_preview_window = ['hidden,right,50%', 'ctrl-/'] ]], true)
vim.api.nvim_exec(
  [[
let g:fzf_layout = {
  \ 'window': {
    \ 'border': 'top',
    \ 'height': 10,
    \ 'relative': v:false,
    \ 'width': 1,
    \ 'yoffset': 1,
  \ }
\ }
let g:fzf_colors =
\ { 'fg':      ['fg', 'Normal'],
\ 'bg':      ['bg', 'Normal'],
\ 'hl':      ['fg', 'Comment'],
\ 'fg+':     ['fg', 'CursorLine', 'CursorColumn', 'Normal'],
\ 'bg+':     ['bg', 'CursorLine', 'CursorColumn'],
\ 'hl+':     ['fg', 'Statement'],
\ 'info':    ['fg', 'PreProc'],
\ 'border':  ['fg', 'Ignore'],
\ 'prompt':  ['fg', 'Conditional'],
\ 'pointer': ['fg', 'Exception'],
\ 'marker':  ['fg', 'Keyword'],
\ 'spinner': ['fg', 'Label'],
\ 'header':  ['fg', 'Comment'] }
let g:fzf_action = {
  \ 'ctrl-t': 'tab split',
  \ 'ctrl-x': 'split',
  \ 'ctrl-v': 'vsplit',
  \ 'ctrl-q': 'fill_quickfix'}
]],
  true
)
